//Atmospheric scattering from cubi
//I modified his model to get Planet and a fake Multiscattering, the original doesn't have that
//https://www.shadertoy.com/view/XlBfRD

vec2 ray_sphere_intersection(vec3 origin, vec3 dir, float radius){
	float B = dot(origin, dir);
	float C = dot(origin, origin) - radius * radius;
	float D = B * B - C;

	vec2 intersection;

	if (D < 0.0){
		intersection = vec2(-1.0, -1.0);
	} else {
		D = sqrt(D);
		intersection = vec2(-B - D, -B + D);
	}

	return intersection;
}

float chapman_approximation(float X, float h, float cosZenith){
    float c = sqrt(X + h);
	float c_exp_h = c * exp(-h);

	if (cosZenith >= 0.0){
		return c_exp_h / (c * cosZenith + 1.0);
	} else {
		float x0 = sqrt(1.0 - cosZenith * cosZenith) * (X + h);
		float c0 = sqrt(x0);

		return 2.0 * c0 * exp(X - x0) - c_exp_h / (1.0 - c * cosZenith);
	}
}

float optical_depth_schueler(float h, float H, float radius, float cos_z){
    return H * chapman_approximation(radius / H, h / H, cos_z);
}

vec3 atmosphere_transmittance(atmosphere_constant ac, vec3 v, vec3 l){
    return exp2(-(ac.wave_lambda_rayleigh + ac.wave_lambda_mie) * optical_depth_schueler(l.y, ac.rayleigh_height, ac.planet_radius, v.y));
}

#include "/program/shaders1/common/atmosphere/phase.glsl"
#include "/program/shaders1/common/atmosphere/object.glsl"

vec2 optical_depth(atmosphere_constant ac, vec3 point, vec3 v, vec3 l, float n){
    float rl = length(point);
    float h = rl -  ac.planet_radius;

    vec3 r = point / rl;

    float cos_chi_sun = dot(r, l);
    float cos_chi_ray = dot(r, v * n);

    float optical_depth_sun = optical_depth_schueler(h, ac.rayleigh_height, ac.planet_radius, cos_chi_sun);
    float optical_depth_ray = optical_depth_schueler(h, ac.rayleigh_height, ac.planet_radius, cos_chi_ray);

    return vec2(optical_depth_sun, optical_depth_ray);
}

void aerial_perspective(atmosphere_constant ac, vec3 start, vec3 end, vec3 v, vec3 l, bool infinite, inout vec3 transmittance, inout vec3 mie, inout vec3 rayleigh){
    float n = infinite ? 1.0 : -1.0;

    vec3 steps = (end - start) / float(ac.steps);
    vec3 point = end - steps;
    vec3 lambda = ac.wave_lambda_rayleigh + ac.wave_lambda_mie + ac.wave_lambda_ozone;

    vec2 last_optical_depth = optical_depth(ac, end, v, l, n);
    vec3 scattering = vec3(0);

    for(int i = 0; i < ac.steps; i++, point -= steps){
        vec2 optical_depth = optical_depth(ac, point, v, l, n);

        vec3 segs = exp2(-lambda * (optical_depth.x + last_optical_depth.x));
        vec3 segt = exp2(-lambda * (optical_depth.y - last_optical_depth.y));

        scattering *= segt;
        scattering += exp2(-(length(point) - ac.planet_radius) / ac.rayleigh_height) * segs;

        transmittance *= segt;
        last_optical_depth = optical_depth;
    }

    rayleigh += scattering * ac.wave_lambda_rayleigh * length(steps);
    mie += scattering * ac.wave_lambda_mie * length(steps);
}
//Niederschaeffolsheim
void skybox_chapman(atmosphere_constant ac, vec3 v, vec3 l, inout vec3 transmittance, inout vec3 mie, inout vec3 rayleigh){
    bool n = true;

    vec3 view_position = vec3(0.0, ac.planet_radius + cameraPosition.y, 0.0);

    vec2 sphere0 = ray_sphere_intersection(view_position, v, ac.atmosphere_radius);
    vec2 sphere1 = ray_sphere_intersection(view_position, v, ac.planet_radius * 0.9999);

    if(sphere1.y >= 0.0){
        n = false;
        sphere0.y = sphere1.x;
    }

    vec3 start = view_position + v * max0(sphere0.x);
    vec3 end = view_position + v * sphere0.y;

    aerial_perspective(ac, start, end, v, l, n, transmittance, mie, rayleigh);

    if(sphere1.y >= 0.0){
        transmittance = vec3(0.0);
    }
}

vec3 atmospheric_scattering(atmosphere_constant ac, vec3 v, vec3 l, inout vec3 transmittance){
    vec3 rayleigh0 = vec3(0.0);
    vec3 rayleigh1 = vec3(0.0);

    vec3 mie0 = vec3(0.0);
    vec3 mie1 = vec3(0.0);

    skybox_chapman(ac, v, l, transmittance, mie0, rayleigh0);
    skybox_chapman(ac, vec3(0.0, 1.0, 0.0), l, transmittance, mie1, rayleigh1);

    float theta = dot(v, l);

    float phase_rayleigh = rayleigh_phase(theta);
    float phase_mie = mie_phase(theta, ac.mie_g);

    vec3 total_rayleigh = (rayleigh0 * phase_rayleigh) + (rayleigh1 * phase_rayleigh);
    vec3 total_mie = (mie0 * phase_mie) + (mie1 * phase_mie);
    
    return pow(total_rayleigh + total_mie, vec3(1.1));
}

vec3 atmospheric_scattering(vec3 position, vec3 sun_position, vec3 moon_position, inout vec3 transmittance){
    atmosphere_constant ac = atmosphere();

    #if defined OVER_WORLD
        #if ATMOSPHERE_TYPE == 0
            vec3 sun_atmosphere = atmospheric_scattering(ac, position, sun_position, transmittance);
                 sun_atmosphere *= ac.sun_luminance * (1.0 + SUNLIGHT_BRIGHTNESS);

            vec3 moon_atmosphere = atmospheric_scattering(ac, position, moon_position, transmittance);
                 moon_atmosphere *= ac.moon_luminance * (1.0 + MOONLIGHT_BRIGHTNESS);

            return max0(sun_atmosphere + moon_atmosphere);
        #elif ATMOSPHERE_TYPE == 1
            return vec3(0.0);
        #endif
    #elif defined END_WORLD || defined NETHER_WORLD
        return vec3(0.0);
    #endif
}

vec3 transmittance_color(vec3 sun_position, vec3 moon_position){
    atmosphere_constant ac = atmosphere();

    const float lod = ATMOSPHERE_LOD;

    vec3 view_position = vec3(0.0, 1.0, 0.0);

    float sun_transition = saturate(dot(view_position, sun_position) * 18.0);
    float moon_transition = saturate(dot(view_position, moon_position) * 18.0);

    #if defined OVER_WORLD
        #if ATMOSPHERE_TYPE == 0
            vec3 sun_transmittance = texture2(colortex7, project_sphere(sun_position) / lod).xyz;
                 sun_transmittance = pow(sun_transmittance, vec3(0.1));

            vec3 moon_transmittance = texture2(colortex7, project_sphere(moon_position) / lod).xyz;
                 moon_transmittance = pow(moon_transmittance, vec3(0.1));

            vec3 sunlight = atmosphere_transmittance(ac, sun_position, sun_position);
                 sunlight *= temperature_to_rgb(5778.0) * ac.sun_luminance * sun_transmittance * sun_transition;
                 sunlight *= 1.0 + SUNLIGHT_BRIGHTNESS;

            vec3 moonlight = atmosphere_transmittance(ac, moon_position, moon_position);
                 moonlight *= ac.moon_luminance * moon_transmittance * moon_transition;
                 moonlight *= 1.0 + MOONLIGHT_BRIGHTNESS;
                
            return max0(sunlight + moonlight);
        #elif ATMOSPHERE_TYPE == 1
            return sunAngle < 0.5 ? ac.sun_luminance * sun_transition : vec3(0.0);
        #endif
    #elif defined END_WORLD
        return ac.sun_luminance * sun_transition;
    #elif defined NETHER_WORLD
        return vec3(0.0);
    #endif
}